﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ConsoleApp1.QMSAPI;
using ServiceKeyClientMessageInspector = ConsoleApp1.QMSClientServiceKeySupport;

namespace ConsoleApp1
{
    class RemoveAssignedCALs
    {
        public void Execute()
        {
            Console.WriteLine("Executing RemoveAssignedCALs...\n");
            string appName = "asdf.qvw";
            try
            {
                // Initiate backend client
                var backendClient = new QMSClient();
                QMSClientServiceKeySupport.Initiate(backendClient);
                
                // Get a time limited service key
                // This is already done in Initiate()

                // Get available QlikView Servers
                var serviceList = backendClient.GetServices(ServiceTypes.QlikViewServer);

                // Loop through available servers
                foreach (var server in serviceList)
                {
                    // Get CAL configuration
                    var config = backendClient.GetCALConfiguration(server.ID, CALConfigurationScope.NamedCALs);

                    // Get number of users BEFORE modifications
                    var numberOfUsers = config.NamedCALs.AssignedCALs.Count<AssignedNamedCAL>();

                    //number of allocated cals (in theory DOCUMENT Cals)
                    int numCalsAlloc = 0;

                    /**********************s
                     *   DOCUMENT CALS
                     **********************/

                    // Get Document CAL's
                    var userDocuments = backendClient.GetUserDocuments(server.ID);

                    foreach (var docNode in userDocuments)
                    {
                        if (!docNode.Name.Equals(appName)) continue;

                        // Get licensing meta data
                        var metaData = backendClient.GetDocumentMetaData(docNode, DocumentMetaDataScope.Licensing);

                        // Get number of users BEFORE modifications
                        numberOfUsers = metaData.Licensing.AssignedCALs.Count<AssignedNamedCAL>();

                        //Get Allocated Cals (we believe DOCUMENT CALS on QMC per document)
                        numCalsAlloc = metaData.Licensing.CALsAllocated;

                        List<AssignedNamedCAL> AssignedCALsList = metaData.Licensing.AssignedCALs.ToList<AssignedNamedCAL>();

                        // Iterate through all CAL's and remove the inactive ones
                        for (int c = 0; c < AssignedCALsList.Count; c++)
                        {
                            if (docNode.Name.Equals(appName))
                            {
                                Console.WriteLine("AssignedCALsList Size before removing : " + AssignedCALsList.Count);
                                Console.WriteLine("DOC CAL to be remove : " + AssignedCALsList[c].UserName + ", " + AssignedCALsList[c].UserName + ", " + AssignedCALsList[c].LastUsed);
                                Console.WriteLine("DOC CAL to be remove : " + AssignedCALsList[c].UserName + ", " + AssignedCALsList[c].UserName + ", " + AssignedCALsList[c].LastUsed);

                                metaData.Licensing.RemovedAssignedCALs.ToList<AssignedNamedCAL>().Add(AssignedCALsList[c]);
                                metaData.Licensing.AssignedCALs.ToList<AssignedNamedCAL>().Remove(AssignedCALsList[c]);


                                Console.WriteLine("AssignedCALsList Size after removing " + AssignedCALsList.Count);
                            }
                        }

                        // Iterate through allocated CAL's (per document) and remove all allocated cals 
                        if (docNode.Name.Equals(appName))
                        {
                            Console.WriteLine("AllocatedCALs total before removing one : " + numCalsAlloc);
                            Console.WriteLine("One DOC CAL to be removed for document: " + docNode);

                            if (numCalsAlloc > 0)
                            {
                                metaData.Licensing.CALsAllocated--;
                                Console.WriteLine("AllocatedCALsList Size after removing one DOC CAL: " + metaData.Licensing.CALsAllocated);
                            }
                            else
                            {
                                Console.WriteLine("No DOC CAL deleted. Total DOC CALS zero.");
                            }
                        }
                        // Save changes
                        backendClient.SaveDocumentMetaData(metaData);
                    }
                }
            }
            catch (SystemException ex)
            {
                Console.WriteLine(ex.Message);
            }
        }
    }
}
